@@ -40,7 +40,8 @@ HierarchicalEmulator <- R6Class(
         self$disc$internal <- ifelse(!is.null(discs$internal), discs$internal, 0)
         self$disc$external <- ifelse(!is.null(discs$external), discs$external, 0)
       }
-      self$beta_u_cov <- function(x) rep(0, length(self$beta_mu))
+      #self$beta_u_cov <- function(x) rep(0, length(self$beta_mu))
+      self$beta_u_cov <- NULL
       if (is.null(ranges)) stop("Ranges for the parameters must be specified.")
       self$ranges <- ranges
       if (!is.null(data)) {
@@ -101,44 +102,79 @@ HierarchicalEmulator <- R6Class(
       else
         beta_part <- predict(self$model, data.frame(x))
       x <- data.matrix(x)
-      bu <- t(apply(x, 1, self$beta_u_cov))
+      if (is.null(self$beta_u_cov)) bu <- matrix(0, nrow = nrow(x), ncol = length(self$beta_mu))
+      else bu <- t(apply(x, 1, self$beta_u_cov))
       u_part <- apply(x, 1, self$u_mu)
       if (!is.null(self$in_data)) {
         if (is.null(c_data))
           c_data <- t(self$corr$get_corr(x, self$in_data, self$active_vars))
         if (is.numeric(self$u_sigma)) {
-          if (length(self$beta_mu) == 1)
-            u_part <- t(u_part + (t(bu) %*% t(private$design_matrix) +
+          if (length(self$beta_mu) == 1) {
+            if (is.null(self$beta_u_cov))
+              u_part <- t(u_part + (self$u_sigma^2 * c_data) %*% private$u_exp_modifier)
+            else
+              u_part <- t(u_part + (t(bu) %*% t(private$design_matrix) +
+                                      self$u_sigma^2 * c_data) %*%
+                            private$u_exp_modifier)
+          }
+          else {
+            if (is.null(self$beta_u_cov))
+              u_part <- u_part + (self$u_sigma^2 * c_data) %*% private$u_exp_modifier
+            else
+              u_part <- u_part + (bu %*% t(private$design_matrix) +
                                     self$u_sigma^2 * c_data) %*%
-                          private$u_exp_modifier)
-          else
-            u_part <- u_part + (bu %*% t(private$design_matrix) +
-                                  self$u_sigma^2 * c_data) %*%
-              private$u_exp_modifier
+                private$u_exp_modifier
+          }
         }
         else {
-          if (length(self$beta_mu) == 1)
-            u_part <- t(u_part + (t(bu) %*% t(private$design_matrix) +
+          if (length(self$beta_mu) == 1) {
+            if (is.null(self$beta_u_cov))
+              u_part <- t(u_part + (
+                                      sweep(
+                                        sweep(
+                                          c_data, 2,
+                                          apply(
+                                            self$in_data, 1,
+                                            self$u_sigma), "*"), 1,
+                                        apply(
+                                          x, 1,
+                                          self$u_sigma), "*")) %*%
+                            private$u_exp_modifier)
+            else
+              u_part <- t(u_part + (t(bu) %*% t(private$design_matrix) +
+                                      sweep(
+                                        sweep(
+                                          c_data, 2,
+                                          apply(
+                                            self$in_data, 1,
+                                            self$u_sigma), "*"), 1,
+                                        apply(
+                                          x, 1,
+                                          self$u_sigma), "*")) %*%
+                            private$u_exp_modifier)
+          }
+          else {
+            if (is.null(self$beta_u_cov))
+              u_part <- u_part + (
                                     sweep(
                                       sweep(
                                         c_data, 2,
                                         apply(
                                           self$in_data, 1,
                                           self$u_sigma), "*"), 1,
-                                      apply(
-                                        x, 1,
-                                        self$u_sigma), "*")) %*%
-                          private$u_exp_modifier)
-          else
-            u_part <- u_part + (bu %*% t(private$design_matrix) +
-                                  sweep(
+                                      apply(x, 1, self$u_sigma), "*")) %*%
+                private$u_exp_modifier
+            else
+              u_part <- u_part + (bu %*% t(private$design_matrix) +
                                     sweep(
-                                      c_data, 2,
-                                      apply(
-                                        self$in_data, 1,
-                                        self$u_sigma), "*"), 1,
-                                    apply(x, 1, self$u_sigma), "*")) %*%
-              private$u_exp_modifier
+                                      sweep(
+                                        c_data, 2,
+                                        apply(
+                                          self$in_data, 1,
+                                          self$u_sigma), "*"), 1,
+                                      apply(x, 1, self$u_sigma), "*")) %*%
+                private$u_exp_modifier
+          }
         }
       }
       if (length(self$beta_mu) == 1) out_val <- c(beta_part + u_part)
@@ -172,7 +208,8 @@ HierarchicalEmulator <- R6Class(
           function(y) map_dbl(self$basis_f, exec, y))
       else g_x <- NULL
       x <- data.matrix(x)
-      bupart_x <- apply(x, 1, self$beta_u_cov)
+      if (is.null(self$beta_u_cov)) bupart_x <- matrix(0, nrow = length(self$beta_mu), ncol = nrow(x))
+      else bupart_x <- apply(x, 1, self$beta_u_cov)
       null_flag <- FALSE
       if (is.null(xp)) {
         null_flag <- TRUE
@@ -190,7 +227,8 @@ HierarchicalEmulator <- R6Class(
             function(y) map_dbl(self$basis_f, exec, y))
         else g_xp <- NULL
         xp <- data.matrix(xp)
-        bupart_xp <- apply(xp, 1, self$beta_u_cov)
+        if (is.null(self$beta_u_cov)) bupart_xp <- matrix(0, nrow = length(self$beta_mu), ncol = nrow(xp))
+        else bupart_xp <- apply(xp, 1, self$beta_u_cov)
       }
       if (full || nrow(x) != nrow(xp)) {
         x_xp_c <- self$corr$get_corr(xp, x, self$active_vars)
@@ -222,14 +260,22 @@ HierarchicalEmulator <- R6Class(
           if (is.numeric(self$u_sigma)) {
             u_part <- u_part - self$u_sigma^4 * c_x %*%
               (private$data_corrs - private$u_var_modifier) %*% t(c_xp)
-            bupart_x <- bupart_x -
-              private$beta_u_cov_modifier %*%
-              (private$design_matrix %*% bupart_x + self$u_sigma^2 * t(c_x))
-            bupart_xp <- if (null_flag)
-              bupart_x
+            if (is.null(self$beta_u_cov))
+              bupart_x <- -private$beta_u_cov_modifier %*%
+                (self$u_sigma^2 * t(c_x))
             else
-              bupart_xp - private$beta_u_cov_modifier %*%
-              (private$design_matrix %*% bupart_xp + self$u_sigma^2 * t(c_xp))
+              bupart_x <- bupart_x -
+                private$beta_u_cov_modifier %*%
+                (private$design_matrix %*% bupart_x + self$u_sigma^2 * t(c_x))
+            if (null_flag) bupart_xp <- bupart_x
+            else {
+              if (is.null(self$beta_u_cov))
+                bupart_xp <- -private$beta_u_cov_modifier %*%
+                  (self$u_sigma^2 * t(c_xp))
+              else
+                bupart_xp <- bupart_xp - private$beta_u_cov_modifier %*%
+                  (private$design_matrix %*% bupart_xp + self$u_sigma^2 * t(c_xp))
+            }
           }
           else {
             c_x <- sweep(
@@ -249,14 +295,20 @@ HierarchicalEmulator <- R6Class(
             u_part <- u_part - c_x %*%
               (private$data_corrs - private$u_var_modifier) %*% t(c_xp)
             if (!all(private$beta_u_cov_modifier == 0)) {
-              bupart_x <- bupart_x -
-                private$beta_u_cov_modifier %*%
-                (private$design_matrix %*% bupart_x + t(c_x))
-              bupart_xp <- if(null_flag)
-                bupart_x
+              if (is.null(self$beta_u_cov))
+                bupart_x <- -private$beta_u_cov_modifier %*% t(c_x)
               else
-                bupart_xp - private$beta_u_cov_modifier %*%
-                (private$design_matrix %*% bupart_xp + t(c_xp))
+                bupart_x <- bupart_x -
+                  private$beta_u_cov_modifier %*%
+                  (private$design_matrix %*% bupart_x + t(c_x))
+              if(null_flag) bupart_xp <- bupart_x
+              else {
+                if (is.null(self$beta_u_cov))
+                  bupart_xp <- -private$beta_u_cov_modifier %*% t(c_xp)
+                else
+                  bupart_xp <- bupart_xp - private$beta_u_cov_modifier %*%
+                  (private$design_matrix %*% bupart_xp + t(c_xp))
+              }
             }
           }
           if (is.null(nrow(g_x))) {
@@ -330,14 +382,20 @@ HierarchicalEmulator <- R6Class(
             rowSums(
               (c_x %*% (private$data_corrs - private$u_var_modifier)) * c_xp)
           if (!all(private$beta_u_cov_modifier == 0)) {
-            bupart_x <- bupart_x -
-              private$beta_u_cov_modifier %*%
-              (private$design_matrix %*% bupart_x + t(c_x))
-            bupart_xp <- if(null_flag)
-              bupart_x
+            if (is.null(self$beta_u_cov))
+                bupart_x <- -private$beta_u_cov_modifier %*% t(c_x)
             else
-              bupart_xp - private$beta_u_cov_modifier %*%
-              (private$design_matrix %*% bupart_xp + t(c_xp))
+              bupart_x <- bupart_x -
+                private$beta_u_cov_modifier %*%
+                (private$design_matrix %*% bupart_x + t(c_x))
+            if(null_flag) bupart_xp <- bupart_x
+            else {
+              if (is.null(self$beta_u_cov))
+                bupart_xp <- -private$beta_u_cov_modifier %*% t(c_xp)
+              else
+                bupart_xp <- bupart_xp - private$beta_u_cov_modifier %*%
+                (private$design_matrix %*% bupart_xp + t(c_xp))
+            }
           }
         }
         if (!all(bupart_x == 0)) {
@@ -586,7 +644,10 @@ HierarchicalEmulator <- R6Class(
             self$u_sigma(map_dbl(self$ranges, mean))^2, "\n")
       cat("\t Expectation: ", self$u_mu(rep(0, length(ranges))), "\n")
       self$corr$print(prepend = "\t")
-      cat("Mixed covariance: ", self$beta_u_cov(rep(0, length(ranges))), "\n")
+      if (is.null(self$beta_u_cov))
+        cat("Mixed covariance: None")
+      else
+        cat("Mixed covariance: ", self$beta_u_cov(rep(0, length(ranges))), "\n")
     }
   )
 )
